package com.ibm.bsf.engines.objrexx;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import com.ibm.bsf.*;
import com.ibm.bsf.util.*;
import com.ibm.bsf.BSFManager;
import com.ibm.bsf.util.BSFEngineImpl;

class JavaObjRexxSAA {
    BSFManager mgr;
    BSFEngine rengine;
//    String EventScript;
    
/************************************************************
 * This class uses JNI to call a dynamic library written in C++
 * that access Rexx through the Rexx SAA API
 ************************************************************/    
    JavaObjRexxSAA(BSFManager mgr, BSFEngine rengine) {
        this.mgr = mgr;         //the current BSFManager
        this.rengine = rengine; //the current RexxEngine
//        EventScript = "";
    }        

    /************************************************************
     * 'RegisterBSF' - a native function to register the 'CallBSF'
     * function with the Rexx interpreter
     ************************************************************/        
    public native long RegisterBSF();
    
    /************************************************************
     * 'Start' - a native function to start evaluating a piece 
     * of Rexx script
     ************************************************************/        
    public native long Start(String script);
    
    /************************************************************
     * a static method to load the dynamic library 'JaReSAA'
     ************************************************************/        
    static {
        System.loadLibrary("JaObjReSAA");
    }     
    
    /************************************************************
     * 'SetEventScript' - a native function to hand an event script 
     * over to Rexx
     ************************************************************/
    public native void SetEventScript(String script);
    
     /************************************************************
      * The 'CallBSF' function is called by Rexx to access the BSF
      * First parameter has to be a function name      
      *       - addEventListener  
      *       - callFunction  
      *       - exit
      *       - pollEvents
      *       - registerBean
      *       - unregisterBean            
      *   Second paramter is a bean name
      *   The following parameters are the 'real' parameters 
      *   for the called function
      * 
      * in some cases argTypes are used these can be:
      * - B: boolean
      * - I: integer
      * - L: long
      * - F: float
      * - D: double
      * - O: bean (object)
      * - S: string
      ************************************************************/
    public String CallBSF(String args[]) throws BSFException{                
        /*********************************************************
         * CallBSF addEventListener beanName eventSetName filter script
         *
         * - beanName: the bean that fires the event
         * - eventSetName: the set the event belongs to
         * - filter: the actual event
         * - script: the script to be run, if the event fires
         *********************************************************/
        if(args[0].equalsIgnoreCase("addEventListener")) {
            if(args.length!=5){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "invalid # of args; usage: CallBSF " +
                    "addEventListener beanName eventSetName filter script)");
            }
            Object bean = mgr.lookupBean(args[1]);
            if(bean==null){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "Bean '"+args[1]+"' not registered");                    
            }            
            args[3] = args[3].equals("") ? null : args[3];
            try {
                EngineUtils.addEventListener(bean, args[2], args[3], 
                                             rengine, mgr, "<event-script>", 
                                             0, 0, args[4]);
            } catch (BSFException e){
                e.printStackTrace ();
                System.out.println("got BSF exception: " + e.getMessage ());
            }            
        }
        /*********************************************************
         * CallBSF callFunction beanName function (argType arg)
         *
         * - beanName: the bean whos function is to be called
         * - function: the function's name
         * - argType: the type of an argument
         * - arg: the argument's value
         *********************************************************/
        if(args[0].equalsIgnoreCase("callFunction")) {
            if((args.length<3) | ((args.length-3)%2!=0)){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "invalid # of args; usage: CallBSF " +
                        "callFunction beanName function (argType arg)");
            }
            Object bean = mgr.lookupBean(args[1]);
            if(bean==null){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "Bean '"+args[1]+"' not registered");                    
            }
            //convert function-arguments to the correct type            
            Object funcArgs[] = new Object[(args.length-3)/2];            
            if(args.length>3){
                for(int i=3, j=0; i<args.length; i+=2, j++){                        
                    switch(args[i].charAt(0)){
                        case 'B':                                 
                            funcArgs[j] = new Boolean(args[i+1]);
                            break;
                        case 'I':
                            funcArgs[j] = new Integer(args[i+1]);
                            break;
                        case 'L':
                            funcArgs[j] = new Long(args[i+1]);
                            break;
                        case 'F':
                            funcArgs[j] = new Float(args[i+1]);
                            break;
                        case 'D':
                            funcArgs[j] = new Double(args[i+1]);
                            break;
                        case 'O':
                            funcArgs[j] = mgr.lookupBean(args[i+1]);
                            if(funcArgs[j]==null){
                                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                                    "Bean '"+args[i+1]+"' not registered");                    
                            } 
                            break;
                        case 'S':
                            funcArgs[j] = args[i+1];
                            break;
                        default:
                            throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                                "argType '"+args[i]+"' unknown");                            
                    }
                }
            }
            //find correct function name
            Class c = bean.getClass();
            Method[] m = c.getMethods();
            for(int i=0; i<m.length; i++) {
                if(args[2].equalsIgnoreCase(m[i].getName())) {
                    args[2]=m[i].getName();
                    i=m.length;
                }
            }
            //call the function
            Object result;
            try {                
                result = EngineUtils.callBeanMethod(bean, args[2], funcArgs);
                if(result==null) {return null;}
                else{
                    return result.toString();
                }
            } catch (BSFException e){
                e.printStackTrace ();
                System.out.println("got BSF exception: " + e.getMessage ());
            }            
        }        
        /*********************************************************
         * CallBSF exit
         * terminates the application         
         *********************************************************/
        if(args[0].equalsIgnoreCase("exit")) {
            System.exit(0);
        }
        /*********************************************************
         * CallBSF registerBean beanName beanType (argType arg)
         *
         * - beanName: the bean's name
         * - beanType: the bean's type
         * - argType: the type of a constructor argument
         * - arg: the argument's value
         *********************************************************/
        if(args[0].equalsIgnoreCase("registerBean")) {
            if((args.length<3) | ((args.length-3)%2!=0)){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "invalid # of args; usage: CallBSF " +
                        "registerBean beanName beanType (argType arg)");
            }            
            //convert arguments to the correct type            
            Object funcArgs[] = new Object[(args.length-3)/2];            
            if(args.length>3){
                for(int i=3, j=0; i<args.length; i+=2, j++){                        
                    switch(args[i].charAt(0)){
                        case 'B':                                 
                            funcArgs[j] = new Boolean(args[i+1]);
                            break;
                        case 'I':
                            funcArgs[j] = new Integer(args[i+1]);
                            break;
                        case 'L':
                            funcArgs[j] = new Long(args[i+1]);
                            break;
                        case 'F':
                            funcArgs[j] = new Float(args[i+1]);
                            break;
                        case 'D':
                            funcArgs[j] = new Double(args[i+1]);
                            break;
                        case 'O':
                            funcArgs[j] = mgr.lookupBean(args[i+1]);
                            if(funcArgs[j]==null){
                                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                                    "Bean '"+args[i+1]+"' not registered");                    
                            } 
                            break;
                        case 'S':
                            funcArgs[j] = args[i+1];
                            break;
                        default:
                            throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                                "argType '"+args[i]+"' unknown");                            
                    }
                }
            }
            //create the bean
            try {                
                Object bean = EngineUtils.createBean(args[2],funcArgs);                
                mgr.registerBean(args[1], bean);
            } catch (BSFException e){
                e.printStackTrace ();
                System.out.println("got BSF exception: " + e.getMessage ());
            }            
        }
        /*********************************************************
         * CallBSF unregisterBean beanName
         *
         * - beanName: the bean to unregister         
         *********************************************************/
        if(args[0].equalsIgnoreCase("unregisterBean")) {            
            if(args.length!=2){
                throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
                    "invalid # of args; usage: CallBSF " +
                    "unregisterBean beanName");
            }
            mgr.unregisterBean(args[1]);
        }                
        return null;
    }        
}
/*
javah -jni -classpath e:\java\bsf\ com.ibm.bsf.engines.objrexx.JavaObjRexxSAA
javap -s -p -classpath e:\java\bsf\ com.ibm.bsf.engines.objrexx.JavaObjRexxSAA
*/